/* Copyright 2008 BORDEAUX I UNIVERSITY & INRIA 
**
** This file is part of the PaStiX parallel sparse matrix package.
**
** This software is governed by the CeCILL-C license under French law
** and abiding by the rules of distribution of free software. You can
** use, modify and/or redistribute the software under the terms of the
** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
** URL: "http://www.cecill.info".
** 
** As a counterpart to the access to the source code and rights to copy,
** modify and redistribute granted by the license, users are provided
** only with a limited warranty and the software's author, the holder of
** the economic rights, and the successive licensors have only limited
** liability.
** 
** In this respect, the user's attention is drawn to the risks associated
** with loading, using, modifying and/or developing or reproducing the
** software by the user in light of its specific status of free software,
** that may mean that it is complicated to manipulate, and that also
** therefore means that it is reserved for developers and experienced
** professionals having in-depth computer knowledge. Users are therefore
** encouraged to load and test the software's suitability as regards
** their requirements in conditions enabling the security of their
** systems and/or data to be ensured and, more generally, to use and
** operate it in the same conditions as regards security.
** 
** The fact that you are presently reading this means that you have had
** knowledge of the CeCILL-C license and that you accept its terms.
*/
/** Structure PaStiX pour une instance du solveur */
struct pastix_data_t;
typedef struct pastix_data_t pastix_data_t;

/*
  Function: pastix

  Computes one to all steps of the resolution of Ax=b linear system, using direct methods.
  
  The matrix is given in CSC format.

  Parameters:
    pastix_data - Data used for a step by step execution.
    pastix_comm - MPI communicator which compute the resolution.
    n           - Size of the system.
    colptr      - Tabular containing the start of each column in row and avals tabulars.
    row         - Tabular containing the row number for each element sorted by column.
    avals       - Tabular containing the values of each elements sorted by column.
    perm        - Permutation tabular for the renumerotation of the unknowns.
    invp        - Reverse permutation tabular for the renumerotation of the unknowns.
    b           - Right hand side vector(s).
    rhs         - Number of right hand side vector(s).
    iparm       - Integer parameters given to pastix.
    dparm       - Double parameters given to pstix.

  About: Example

    from file <simple.c> :

    > /\*******************************************\/
    > /\*    Check Matrix format                  *\/
    > /\*******************************************\/
    > /\*                                      
    >  * Matrix needs :
    >  *    - to be in fortran numbering
    >  *    - to have only the lower triangular part in symmetric case
    >  *    - to have a graph with a symmetric structure in unsymmetric case
    >  *\/
    > pastix_checkMatrix(MPI_COMM_WORLD, verbosemode, 
    >       	     (MTX_ISSYM(type) ? API_SYM_YES : API_SYM_NO),  API_YES,
    >       	     ncol, &colptr, &rows, &values, NULL);
    > 
    > /\*******************************************\/
    > /\* Initialize parameters to default values *\/
    > /\*******************************************\/
    > iparm[IPARM_MODIFY_PARAMETER] = API_NO;
    > pastix(&pastix_data, MPI_COMM_WORLD, 
    >        ncol, colptr, rows, values, 
    >        perm, invp, rhs, 1, iparm, dparm);

    > /\*******************************************\/
    > /\*       Customize some parameters         *\/
    > /\*******************************************\/
    > iparm[IPARM_THREAD_NBR] = nbthread;
    > if (MTX_ISSYM(type)) {
    >   :wiparm[IPARM_SYM]           = API_SYM_YES;
    >   iparm[IPARM_FACTORIZATION] = API_FACT_LDLT;
    > }
    > else{
    >   iparm[IPARM_SYM]           = API_SYM_NO;
    >   iparm[IPARM_FACTORIZATION] = API_FACT_LU;
    > }
    > iparm[IPARM_START_TASK]          = API_TASK_ORDERING;
    > iparm[IPARM_END_TASK]            = API_TASK_CLEAN;

    > /\*******************************************\/
    > /\*           Save the rhs                  *\/
    > /\*    (it will be replaced by solution)    *\/
    > /\*******************************************\/
    > rhssaved = malloc(ncol*sizeof(pastix_float_t));
    > memcpy(rhssaved, rhs, ncol*sizeof(pastix_float_t));

    > /\*******************************************\/
    > /\*           Call pastix                   *\/
    > /\*******************************************\/
    > perm = malloc(ncol*sizeof(pastix_int_t));
    > invp = malloc(ncol*sizeof(pastix_int_t));

    > pastix(&pastix_data, MPI_COMM_WORLD, 
    >	 ncol, colptr, rows, values,
    >	 perm, invp, rhs, 1, iparm, dparm);
*/
void pastix(pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
	    FLOAT *avals, INT *perm, INT *invp, FLOAT *b, INT rhs, INT *iparm, double *dparm);

/** Function: dpastix

    Computes one to all steps of the resolution of Ax=b linear system, using direct methods.
    Here the matrix is given distributed.

    The matrix is given in CSCD format.

    Parameters: 
       pastix_data - Data used for a step by step execution.
       pastix_comm - MPI communicator which compute the resolution.
       n           - Size of the system.
       colptr      - Tabular containing the start of each column in row and avals tabulars.
       row         - Tabular containing the row number for each element sorted by column.
       avals       - Tabular containing the values of each elements sorted by column.
       loc2glob    - Global column number of the local columns.
       perm        - Permutation tabular for the renumerotation of the unknowns.
       invp        - Reverse permutation tabular for the renumerotation of the unknowns.
       b           - Right hand side vector(s).
       rhs         - Number of right hand side vector(s).
       iparm       - Integer parameters given to pastix.
       dparm       - Double parameters given to pstix.
 */

void dpastix(pastix_data_t **pastix_data, MPI_Comm pastix_comm, INT n, INT *colptr, INT *row, 
	     FLOAT *avals, INT * loc2glob, INT *perm, INT *invp, FLOAT *b, INT rhs, INT *iparm, double *dparm);

/*
  Function: pastix_setBind

  Set bindtab in pastix_data, it gives for each thread the CPU to bind in to.
  bindtab follows this organisation :

  bindtab[threadnum] = cpu to set thread threadnum.

  Parameters:
    pastix_data - Structure de donne pour l'utilisation step by step
    thrdnbr     - Nombre de threads / Taille du tableau
    bindtab     - Tableau de correspondance entre chaque thread et coeur de la machine
*/
void pastix_setBind(pastix_data_t *pastix_data, int thrdnbr, int *bindtab);

/*
  Function: pastix_checkMatrix 

  Check the matrix :
    - Renumbers in Fortran numerotation (base 1) if needed (base 0)
    - Can scale the matrix if compiled with -DMC64 -DSCALING (untested)
    - Checks the symetry of the graph in non symmetric mode.
      With non distributed matrices, with flagcor == API_YES,
      tries to correct the matrix.
    - sort the CSC.

  Parameters:
    pastix_comm -  PaStiX MPI communicator
    verb        -  Level of prints (API_VERBOSE_[NOT|NO|YES])
    flagsym     -  Indicate if the given matrix is symetric (API_SYM_YES or API_SYM_NO)
    flagcor     -  Indicate if we permit the function to reallocate the matrix.
    n           -  Size of the matrix.
    colptr      -  First element of each row in *row* and *avals*.
    row         -  Row of each element of the matrix.
    avals       -  Value of each element of the matrix.
    loc2glob    -  Global column number of local columns.
    dof         - Number of degrees of freedom.
*/
int pastix_checkMatrix(MPI_Comm pastix_comm, int verb, int flagsym, int flagcor,
		       INT n, INT **colptr, INT **row, FLOAT **avals, INT **loc2glob, int dof);

/*
  Function: pastix_getLocalNodeNbr

  Return the node number in the new distribution computed by blend.
  Needs blend to be runned with pastix_data before.

  Parameters:
    pastix_data - Data used for a step by step execution.

  Returns:
    Number of local nodes/columns in new distribution.
 */
INT pastix_getLocalNodeNbr(pastix_data_t ** pastix_data);

/*
  Function: pastix_getLocalNodeNbr

  Fill in nodelst with the list of local nodes/clumns.
  Needs nodelst to be allocated with nodenbr*sizeof(pastix_int_t), 
  where nodenbr has been computed by <pastix_getLocalNodeNbr>. 

  Parameters:
    pastix_data - Data used for a step by step execution.
    nodelst     - An array where to write the list of local nodes/columns.
 */
int pastix_getLocalNodeLst(pastix_data_t ** pastix_data, INT * nodelst);
